// 2005-11-19 programmeerimisvistlus
// lesanne: TEEDEKAART
// Autor: Kristo Tammeoja
// Testitud: GCC 3.2 / MinGW

// Kasutab Dijkstra algoritmi
// Ajaline keerukus N*M*logM

#include <stdio.h>
#include <time.h>

// 2*SUUR_ARV mahub veel int-i ra
#define SUUR_ARV 1000000000

#define MAX_N 1000
#define MAX_M 10000

struct T_SERV {
    int v1, v2;
    int d;
};

int N, M;

int servi;
T_SERV serv[2*MAX_M];

int serv_v1_esimene[MAX_N+2];
T_SERV serv_v1[2*MAX_M];

int min_d[MAX_N+1];
int max_d[MAX_N+1];

int kaugus[MAX_N+1];

// esimene indeks on 1
int kuhi_suurus;
int kuhi[MAX_N+1];
int kuhi2tipp[MAX_N+1];
int tipp2kuhi[MAX_N+1];

inline void kuhi_alla(int i) {
    int voti = kuhi[i];
    int tipp = kuhi2tipp[i];
    
    int j = i / 2;
    while (j > 0 && voti < kuhi[j]) {
       kuhi[i] = kuhi[j];
       kuhi2tipp[i] = kuhi2tipp[j];
       tipp2kuhi[kuhi2tipp[i]] = i;
       i = j;
       j = i / 2;
    }

    kuhi[i] = voti;
    kuhi2tipp[i] = tipp;
    tipp2kuhi[tipp] = i;
}

inline void kuhi_yles(int i) {
    int voti = kuhi[i];
    int tipp = kuhi2tipp[i];
    
    int j, k;
    while (true) {
       k = 2*i;
       if (k <= kuhi_suurus) j = k; else break;
       k++;
       if (k <= kuhi_suurus && kuhi[k] < kuhi[j]) j = k;
       if (kuhi[j] >= voti) break;
       
       kuhi[i] = kuhi[j];
       kuhi2tipp[i] = kuhi2tipp[j];
       tipp2kuhi[kuhi2tipp[i]] = i;
       i = j;
    }

    kuhi[i] = voti;
    kuhi2tipp[i] = tipp;
    tipp2kuhi[tipp] = i;
}

inline bool kuhi_vahenda_votit(int tipp, int uus_voti) {
    int i = tipp2kuhi[tipp];
    if (uus_voti >= kuhi[i]) return false;
    
    kuhi[i] = uus_voti;
    kuhi_alla(i);
    return true;
}

void kuhi_lisa(int tipp, int voti) {
    kuhi_suurus++;
    kuhi[kuhi_suurus] = voti;
    kuhi2tipp[kuhi_suurus] = tipp;
    tipp2kuhi[tipp] = kuhi_suurus;
    kuhi_alla(kuhi_suurus);
}

int kuhi_vahim_tipp() {
    int tipp = kuhi2tipp[1];
    tipp2kuhi[tipp] = -1;
    
    kuhi[1] = kuhi[kuhi_suurus];
    kuhi2tipp[1] = kuhi2tipp[kuhi_suurus];
    tipp2kuhi[kuhi2tipp[kuhi_suurus]] = 1;

    kuhi_suurus--;
    kuhi_yles(1);

    return tipp;
}

inline bool kuhi_pole_tyhi() {
    return kuhi_suurus > 0;
}

void kuhi_init() {
    kuhi_suurus = 0;
    for (int i = 1; i <= N; i++) tipp2kuhi[i] = -1;
}

void loe_teed() {
    FILE *sisf;
    sisf = fopen("teed.sis", "rt");

    fscanf(sisf, "%d %d", &N, &M);

    int l_1, l_2, d;
    servi = 0;
    for (int e = 0; e < M; e++) {
        fscanf(sisf, "%d %d %d", &l_1, &l_2, &d);
        serv[servi].v1 = l_1;
        serv[servi].v2 = l_2;
        serv[servi].d = d;
        servi++;
        serv[servi].v1 = l_2;
        serv[servi].v2 = l_1;
        serv[servi].d = d;
        servi++;
    }

    for (int i = 1; i <= N+1; i++) serv_v1_esimene[i] = 0;
    for (int e = 0; e < servi; e++) serv_v1_esimene[serv[e].v1]++;
    for (int i = 2; i <= N+1; i++) serv_v1_esimene[i] += serv_v1_esimene[i-1];
    for (int e = 0; e < servi; e++) {
        serv_v1[--serv_v1_esimene[serv[e].v1]] = serv[e];
    }

    fclose(sisf);
}

void arvuta_kaugused(int a) {
    for (int i = 1; i <= N; i++) kaugus[i] = SUUR_ARV;
    kaugus[a] = 0;
    for (int i = 1; i <= N; i++) kuhi_lisa(i, kaugus[i]);
    
    while (kuhi_pole_tyhi()) {
        int lahim_tipp = kuhi_vahim_tipp();
        int lahim_kaugus = kaugus[lahim_tipp];
        kaugus[lahim_tipp] = lahim_kaugus;
        int e_serv = serv_v1_esimene[lahim_tipp];
        int v_serv = serv_v1_esimene[lahim_tipp+1];
        for (int e = e_serv; e < v_serv; e++) {
            int v2 = serv_v1[e].v2;
            if (tipp2kuhi[v2] < 0) continue;
            if (kuhi_vahenda_votit(v2, lahim_kaugus + serv_v1[e].d)) {
                kaugus[v2] = lahim_kaugus + serv_v1[e].d;
            }
        }
    }
}

void leia_vastus() {
    for (int l = 1; l <= N; l++) {
        arvuta_kaugused(l);
        int min_kaugus = SUUR_ARV;
        int max_kaugus = -1;
        for (int i = 1; i <= N; i++) {
            if (i == l) continue;
            int kaugus_i = kaugus[i];
            if (kaugus_i < min_kaugus) min_kaugus = kaugus_i;
            if (max_kaugus < kaugus_i) max_kaugus = kaugus_i;
        }
        min_d[l] = min_kaugus;
        max_d[l] = max_kaugus;
    }
}

void kirjuta_vastus() {
    FILE *valf;
    valf = fopen("teed.val", "wt");
    for (int l = 1; l <= N; l++) {
if (max_d[l] >= SUUR_ARV) fprintf(stdout, "teede graaf pole sidus\n");
        fprintf(valf, "%d %d\n", min_d[l], max_d[l]);
    }
    fclose(valf);
}

int main() {
    loe_teed();
    leia_vastus();
    kirjuta_vastus();
//printf("aeg = %f\n", (float)clock() / CLK_TCK);
    return 0;
}
